@@ -15,30 +15,38 @@
 """
 __docformat__ = 'restructuredtext'
 
-import sys
 import os
+import sys
 import time
 
+
 try:
     import locale
-except ImportError:
+except ImportError:  # pragma: no cover
     locale = None
 
 import platform
 
-from zope.component import getUtility, ComponentLookupError, adapter
+from zope.component import ComponentLookupError
+from zope.component import adapter
+from zope.component import getUtility
 from zope.interface import implementer
 
-from zope.applicationcontrol.interfaces import IRuntimeInfo
 from zope.applicationcontrol.interfaces import IApplicationControl
+from zope.applicationcontrol.interfaces import IRuntimeInfo
 from zope.applicationcontrol.interfaces import IZopeVersion
 
+
 try:
     from zope.app.appsetup import appsetup
 except ImportError:
     appsetup = None
 
-PY3 = sys.version_info[0] == 3
+try:
+    text_type = unicode
+except NameError:
+    text_type = str
+
 
 @implementer(IRuntimeInfo)
 @adapter(IApplicationControl)
@@ -53,8 +61,8 @@ class RuntimeInfo(object):
         if appsetup is None:
             return 'undefined'
 
-        cc=appsetup.getConfigContext()
-        if cc == None:  # make the test run
+        cc = appsetup.getConfigContext()
+        if cc is None:  # make the test run
             return 'undefined'
         if cc.hasFeature('devmode'):
             return 'On'
@@ -62,19 +70,19 @@ class RuntimeInfo(object):
 
     def getPreferredEncoding(self):
         """See zope.app.applicationcontrol.interfaces.IRuntimeInfo"""
-        if locale is not None:
-            try:
-                return locale.getpreferredencoding()
-            except locale.Error:
-                pass
-        return sys.getdefaultencoding()
+        try:
+            result = locale.getpreferredencoding()
+        except (locale.Error, AttributeError):  # pragma: no cover
+            result = ''
+        # Under some systems, getpreferredencoding() can return ''
+        # (e.g., Python 2.7/MacOSX/LANG=en_us.UTF-8). This then blows
+        # up with 'unknown encoding'
+        return result or sys.getdefaultencoding()
 
     def getFileSystemEncoding(self):
         """See zope.app.applicationcontrol.interfaces.IRuntimeInfo"""
         enc = sys.getfilesystemencoding()
-        if enc is None:
-            enc = self.getPreferredEncoding()
-        return enc
+        return enc if enc is not None else self.getPreferredEncoding()
 
     def getZopeVersion(self):
         """See zope.app.applicationcontrol.interfaces.IRuntimeInfo"""
@@ -86,13 +94,15 @@ class RuntimeInfo(object):
 
     def getPythonVersion(self):
         """See zope.app.applicationcontrol.interfaces.IRuntimeInfo"""
-        return sys.version if PY3 else sys.version.decode(
-            self.getPreferredEncoding())
+        return (
+            sys.version
+            if isinstance(sys.version, text_type)
+            else sys.version.decode(self.getPreferredEncoding()))
 
     def getPythonPath(self):
         """See zope.app.applicationcontrol.interfaces.IRuntimeInfo"""
         enc = self.getFileSystemEncoding()
-        return tuple([path if PY3 else path.decode(enc)
+        return tuple([path if isinstance(path, text_type) else path.decode(enc)
                       for path in sys.path])
 
     def getSystemPlatform(self):
@@ -101,16 +111,19 @@ class RuntimeInfo(object):
         enc = self.getPreferredEncoding()
         for item in platform.uname():
             try:
-                t = item if PY3 else item.decode(enc)
-            except ValueError:
+                t = item if isinstance(item, text_type) else item.decode(enc)
+            except UnicodeError:
                 continue
             info.append(t)
-        return u" ".join(info)
+        return u' '.join(info)
 
     def getCommandLine(self):
         """See zope.app.applicationcontrol.interfaces.IRuntimeInfo"""
         cmd = " ".join(sys.argv)
-        return cmd if PY3 else cmd.decode(self.getPreferredEncoding())
+        return (
+            cmd
+            if isinstance(cmd, text_type)
+            else cmd.decode(self.getPreferredEncoding()))
 
     def getProcessId(self):
         """See zope.app.applicationcontrol.interfaces.IRuntimeInfo"""
