@@ -12,16 +12,19 @@
 ##############################################################################
 """Application Control Tests
 """
+import time
 import unittest
+
 from zope.interface.verify import verifyObject
 
-import time
 from zope.applicationcontrol.applicationcontrol import ApplicationControl
 from zope.applicationcontrol.interfaces import IApplicationControl
 
+
 # seconds, time values may differ in order to be assumed equal
 time_tolerance = 2
 
+
 class Test(unittest.TestCase):
 
     def _Test__new(self):
@@ -33,14 +36,4 @@ class Test(unittest.TestCase):
     def test_startTime(self):
         assert_time = time.time()
         test_time = self._Test__new().getStartTime()
-
-        self.assertTrue(abs(assert_time - test_time) < time_tolerance)
-
-
-def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(Test),
-        ))
-
-if __name__ == '__main__':
-    unittest.main()
+        self.assertAlmostEqual(assert_time, test_time, delta=time_tolerance)
