@@ -12,25 +12,35 @@
 ##############################################################################
 """Runtime Info Tests
 """
+import os
+import sys
+import time
 import unittest
-import os, sys, time
+
 
 try:
     import locale
-except ImportError:
+except ImportError:  # pragma: no cover
     locale = None
 
-from zope import component
 from zope.interface import implementer
 from zope.interface.verify import verifyObject
+
+from zope import component
 from zope.applicationcontrol.applicationcontrol import applicationController
-from zope.applicationcontrol.interfaces import IRuntimeInfo, IZopeVersion
+from zope.applicationcontrol.interfaces import IRuntimeInfo
+from zope.applicationcontrol.interfaces import IZopeVersion
+
 
 # seconds, time values may differ in order to be assumed equal
 time_tolerance = 2
 stupid_version_string = "3085t0klvn93850voids"
 
-PY3 = sys.version_info[0] == 3
+try:
+    text_type = unicode
+except NameError:
+    text_type = str
+
 
 @implementer(IZopeVersion)
 class TestZopeVersion(object):
@@ -47,18 +57,18 @@ class Test(unittest.TestCase):
         return RuntimeInfo(applicationController)
 
     def _getPreferredEncoding(self):
-        if locale is not None:
-            try:
-                return locale.getpreferredencoding()
-            except locale.Error:
-                pass
-        return sys.getdefaultencoding()
+        try:
+            result = locale.getpreferredencoding()
+        except (locale.Error, AttributeError):  # pragma: no cover
+            result = ''
+        # Under some systems, getpreferredencoding() can return ''
+        # (e.g., Python 2.7/MacOSX/LANG=en_us.UTF-8). This then blows
+        # up with 'unknown encoding'
+        return result or sys.getdefaultencoding()
 
     def _getFileSystemEncoding(self):
         enc = sys.getfilesystemencoding()
-        if enc is None:
-            enc = self._getPreferredEncoding()
-        return enc
+        return enc or self._getPreferredEncoding()
 
     def testIRuntimeInfoVerify(self):
         verifyObject(IRuntimeInfo, self._Test__new())
@@ -89,11 +99,34 @@ class Test(unittest.TestCase):
         enc = self._getPreferredEncoding()
         self.assertEqual(
             runtime_info.getPythonVersion(),
-            sys.version if PY3 else sys.version.decode(enc))
+            sys.version if not isinstance(sys.version, bytes)
+            else sys.version.decode(enc))
 
     def test_SystemPlatform(self):
         runtime_info = self._Test__new()
-        self.assertTrue(runtime_info.getSystemPlatform())
+        plat = runtime_info.getSystemPlatform()
+        self.assertTrue(plat)
+        self.assertIsInstance(plat, text_type)
+
+        # Now something that can't be decoded
+        import platform
+        uname = platform.uname
+
+        def bad_uname():
+            class BadObject(object):
+                def decode(self, _enc):
+                    raise UnicodeError("Not gonna happen")
+            return (BadObject(),)
+        try:
+            platform.uname = bad_uname
+            plat = runtime_info.getSystemPlatform()
+            self.assertEqual(plat, u'')
+
+            platform.uname = lambda: ('a', 'b')
+            plat = runtime_info.getSystemPlatform()
+            self.assertEqual(plat, u'a b')
+        finally:
+            platform.uname = uname
 
     def test_CommandLine(self):
         runtime_info = self._Test__new()
@@ -112,14 +145,51 @@ class Test(unittest.TestCase):
 
         # get the uptime the current implementation calculates
         test_uptime = runtime_info.getUptime()
+        self.assertAlmostEqual(
+            asserted_uptime, test_uptime, delta=time_tolerance)
 
-        self.assertTrue(abs(asserted_uptime - test_uptime) < time_tolerance)
+    @unittest.skipIf(not sys.path, "Need entries on path")
+    def test_getPythonPath_returns_unicode(self):
+        runtime_info = self._Test__new()
+        path = runtime_info.getPythonPath()
+        for p in path:
+            self.assertIsInstance(p, text_type)
 
+    def test_getDeveloperMode(self):
+        from zope.applicationcontrol import runtimeinfo as module
+        orig_appsetup = module.appsetup
 
-def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(Test),
-        ))
+        runtime_info = self._Test__new()
 
-if __name__ == '__main__':
-    unittest.main()
+        try:
+            # Module not available
+            module.appsetup = None
+            self.assertEqual(runtime_info.getDeveloperMode(), 'undefined')
+
+            # context not available
+            class Setup(object):
+                context = None
+
+                @classmethod
+                def getConfigContext(cls):
+                    return cls.context
+
+            module.appsetup = Setup
+            self.assertEqual(runtime_info.getDeveloperMode(), 'undefined')
+
+            features = []
+
+            class Context(object):
+                @staticmethod
+                def hasFeature(name):
+                    return name in features
+
+            # Feature off
+            Setup.context = Context
+            self.assertEqual(runtime_info.getDeveloperMode(), 'Off')
+
+            # Feature on
+            features.append('devmode')
+            self.assertEqual(runtime_info.getDeveloperMode(), 'On')
+        finally:
+            module.appsetup = orig_appsetup
